Package: matrixStats
Version: 0.61.0
Depends: R (>= 2.12.0)
Suggests: base64enc, ggplot2, knitr, markdown, microbenchmark,
        R.devices, R.rsp
VignetteBuilder: R.rsp
Title: Functions that Apply to Rows and Columns of Matrices (and to
        Vectors)
Authors@R: c(
  person("Henrik", "Bengtsson", role=c("aut", "cre", "cph"),
                                 email="henrikb@braju.com"),
  person("Constantin", "Ahlmann-Eltze", role = "ctb"),
  person("Hector", "Corrada Bravo", role="ctb"),
  person("Robert", "Gentleman", role="ctb"),
  person("Jan", "Gleixner", role="ctb"),
  person("Peter", "Hickey", role="ctb"),
  person("Ola", "Hossjer", role="ctb"),
  person("Harris", "Jaffee", role="ctb"),
  person("Dongcan", "Jiang", role="ctb"),
  person("Peter", "Langfelder", role="ctb"),
  person("Brian", "Montgomery", role="ctb"),
  person("Angelina", "Panagopoulou", role="ctb"),
  person("Hugh", "Parsonage", role="ctb"),
  person("Jakob Peder", "Pettersen", role="ctb"))
Author: Henrik Bengtsson [aut, cre, cph],
  Constantin Ahlmann-Eltze [ctb],
  Hector Corrada Bravo [ctb],
  Robert Gentleman [ctb],
  Jan Gleixner [ctb],
  Peter Hickey [ctb],
  Ola Hossjer [ctb],
  Harris Jaffee [ctb],
  Dongcan Jiang [ctb],
  Peter Langfelder [ctb],
  Brian Montgomery [ctb],
  Angelina Panagopoulou [ctb],
  Hugh Parsonage [ctb],
  Jakob Peder Pettersen [ctb]
Maintainer: Henrik Bengtsson <henrikb@braju.com>
Description: High-performing functions operating on rows and columns of matrices, e.g. col / rowMedians(), col / rowRanks(), and col / rowSds().  Functions optimized per data type and for subsetted calculations such that both memory usage and processing time is minimized.  There are also optimized vector-based methods, e.g. binMeans(), madDiff() and weightedMedian().
License: Artistic-2.0
LazyLoad: TRUE
NeedsCompilation: yes
ByteCompile: TRUE
URL: https://github.com/HenrikBengtsson/matrixStats
BugReports: https://github.com/HenrikBengtsson/matrixStats/issues
RoxygenNote: 7.1.2
Packaged: 2021-09-15 17:47:41 UTC; hb
Repository: CRAN
Date/Publication: 2021-09-17 11:10:02 UTC
Built: R 4.0.5; x86_64-w64-mingw32; 2021-09-17 14:19:58 UTC; windows
Archs: x64
