# Generated by roxygen2: do not edit by hand

export(allValue)
export(allocArray)
export(allocMatrix)
export(allocVector)
export(anyMissing)
export(anyValue)
export(binCounts)
export(binMeans)
export(colAlls)
export(colAnyMissings)
export(colAnyNAs)
export(colAnys)
export(colAvgsPerRowSet)
export(colCollapse)
export(colCounts)
export(colCummaxs)
export(colCummins)
export(colCumprods)
export(colCumsums)
export(colDiffs)
export(colIQRDiffs)
export(colIQRs)
export(colLogSumExps)
export(colMadDiffs)
export(colMads)
export(colMaxs)
export(colMeans2)
export(colMedians)
export(colMins)
export(colOrderStats)
export(colProds)
export(colQuantiles)
export(colRanges)
export(colRanks)
export(colSdDiffs)
export(colSds)
export(colSums2)
export(colTabulates)
export(colVarDiffs)
export(colVars)
export(colWeightedMads)
export(colWeightedMeans)
export(colWeightedMedians)
export(colWeightedSds)
export(colWeightedVars)
export(count)
export(diff2)
export(indexByRow)
export(iqr)
export(iqrDiff)
export(logSumExp)
export(madDiff)
export(mean2)
export(meanOver)
export(product)
export(rowAlls)
export(rowAnyMissings)
export(rowAnyNAs)
export(rowAnys)
export(rowAvgsPerColSet)
export(rowCollapse)
export(rowCounts)
export(rowCummaxs)
export(rowCummins)
export(rowCumprods)
export(rowCumsums)
export(rowDiffs)
export(rowIQRDiffs)
export(rowIQRs)
export(rowLogSumExps)
export(rowMadDiffs)
export(rowMads)
export(rowMaxs)
export(rowMeans2)
export(rowMedians)
export(rowMins)
export(rowOrderStats)
export(rowProds)
export(rowQuantiles)
export(rowRanges)
export(rowRanks)
export(rowSdDiffs)
export(rowSds)
export(rowSums2)
export(rowTabulates)
export(rowVarDiffs)
export(rowVars)
export(rowWeightedMads)
export(rowWeightedMeans)
export(rowWeightedMedians)
export(rowWeightedSds)
export(rowWeightedVars)
export(sdDiff)
export(signTabulate)
export(sum2)
export(sumOver)
export(t_tx_OP_y)
export(validateIndices)
export(varDiff)
export(weightedMad)
export(weightedMean)
export(weightedMedian)
export(weightedSd)
export(weightedVar)
export(x_OP_y)
importFrom(stats,mad)
importFrom(stats,median)
importFrom(stats,quantile)
useDynLib("matrixStats", .registration = TRUE, .fixes = "C_")
